@@ -1,4 +1,4 @@
-/*  $Id: ConfigFile.cpp,v 1.23 2006/03/01 14:02:51 terpstra Exp $
+/*  $Id: ConfigFile.cpp 1649 2009-10-19 14:35:01Z terpstra $
  *  
  *  ConfigFile.cpp - Knows how to load the config file
  *  
@@ -26,9 +26,11 @@
 
 #include "ConfigFile.h"
 #include "XmlEscape.h"
+#include "Summary.h"
 
 #include <fstream>
 #include <iostream>
+#include <algorithm>
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
@@ -37,9 +39,13 @@
 #include <unistd.h>
 #include <time.h>
 #include <stdlib.h>
+#include <dirent.h>
+#include <cstring>
 
 using namespace std;
 
+#define ERROR	error << file << ":" << c << ": "
+
 map<string, string>* lstring::c = 0;
 
 lstring::lstring(const string& fallback)
@@ -381,6 +387,7 @@ Config::Config()
  : list(0), frontend(0), group(""), error(), lists(), groups(),
    file(""),
    dbdir(""), 
+   db_umask(-1),
    xslt("cat -"),
    delete_message(""),
    pgpv_mime("off"),
@@ -473,7 +480,7 @@ int Config::load(const string& file, bool toplevel)
 		{	// this line continues the previous one.
 			if (key == "")
 			{
-				error << "No key for value '" << line << "'!" << endl;
+				ERROR << "No key for value '" << line << "'!" << endl;
 				ok = false;
 			}
 			else
@@ -485,7 +492,7 @@ int Config::load(const string& file, bool toplevel)
 		}
 		else
 		{
-			if (key != "" && process_command(key, val, dir) != 0) ok = false;
+			if (key != "" && process_command(file, c, key, val, dir) != 0) ok = false;
 			
 			string::size_type leadin = skip_front(line);
 			key.assign(line, leadin, eq-leadin);
@@ -496,11 +503,11 @@ int Config::load(const string& file, bool toplevel)
 	
 	if (toplevel && key == "")
 	{
-		error << "No values set by config file '" << file << "'!" << endl;
+		error << file << ":eof: No values set!" << endl;
 		ok = false;
 	}
 	
-	if (key != "" && process_command(key, val, dir) != 0) ok = false;
+	if (key != "" && process_command(file, c, key, val, dir) != 0) ok = false;
 	
 	if (toplevel)
 	{
@@ -512,7 +519,7 @@ int Config::load(const string& file, bool toplevel)
 		{
 			if (i->second.languages.empty()) 
 			{
-				error << "List '" << i->first << "' has no language!" << endl;
+				error << file << ":eof: List '" << i->first << "' has no language!" << endl;
 				return -1;
 			}
 		}
@@ -530,7 +537,7 @@ int Config::load(const string& file, bool toplevel)
 			
 			if (fi->first + "/" == fn->first.substr(0, fi->first.length()+1))
 			{
-				error << "Frontend '" << fi->first << "' is a prefix of '" << fn->first << "', which is forbidden!" << endl;
+				error << file << ":eof: Frontend '" << fi->first << "' is a prefix of '" << fn->first << "', which is forbidden!" << endl;
 				return -1;
 			}
 			
@@ -559,25 +566,25 @@ bool isSimple(const string& s)
 	return true;
 }
 
-int Config::process_command(const string& keys, const string& val, const string& dir)
+int Config::process_command(const string& file, int c, const string& keys, const string& val, const string& dir)
 {
 //	cout << key << "-" << val << endl;
 	
 	string lc; // locale code
 	string key(keys);
 	
-	string::size_type o, c;
+	string::size_type o, d;
 	if ((o = key.find('[')) != string::npos &&
-	    (c = key.find(']')) != string::npos &&
-	    c > o)
+	    (d = key.find(']')) != string::npos &&
+	    d > o)
 	{
 		// localization option
-		lc.assign(key, o+1, (c-o) - 1);
-		key.erase(o, (c-o) + 1);
+		lc.assign(key, o+1, (d-o) - 1);
+		key.erase(o, (d-o) + 1);
 		
 		if (!lstring::locale_normalize(lc))
 		{
-			error << "Localization code '" << lc << "' is not valid." << endl;
+			ERROR << "Localization code '" << lc << "' is not valid." << endl;
 			return -1;
 		}
 	}
@@ -589,19 +596,19 @@ int Config::process_command(const string& keys, const string& val, const string&
 		len = 128;
 		if (!isSimple(val) || val.length() == 0)
 		{
-			error << "Group id '" << val << "' is not a simple lowercase string!" << endl;
+			ERROR << "Group id '" << val << "' is not a simple lowercase string!" << endl;
 			return -1;
 		}
 		
 		if (lc != "")
 		{
-			error << "group id cannot be localized" << endl;
+			ERROR << "group id cannot be localized" << endl;
 			return -1;
 		}
 		
 		if (groups.find(val) != groups.end())
 		{
-			error << "Group id '" << val << "' already exists!" << endl;
+			ERROR << "Group id '" << val << "' already exists!" << endl;
 			return -1;
 		}
 		
@@ -617,19 +624,19 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (group == "")
 		{
-			error << "List id '" << val << "' is not a member of any group!" << endl;
+			ERROR << "List id '" << val << "' is not a member of any group!" << endl;
 			return -1;
 		}
 		
 		len = 128;
 		if (!isSimple(val) || val.length() == 0)
 		{
-			error << "List id '" << val << "' is not a simple lowercase string!" << endl;
+			ERROR << "List id '" << val << "' is not a simple lowercase string!" << endl;
 			return -1;
 		}
 		if (lc != "")
 		{
-			error << "list id cannot be localized" << endl;
+			ERROR << "list id cannot be localized" << endl;
 			return -1;
 		}
 		
@@ -643,7 +650,7 @@ int Config::process_command(const string& keys, const string& val, const string&
 		}
 		else
 		{
-			error << "List id '" << val << "' already exists!" << endl;
+			ERROR << "List id '" << val << "' already exists!" << endl;
 			return -1;
 		}
 	}
@@ -653,7 +660,7 @@ int Config::process_command(const string& keys, const string& val, const string&
 		
 		if (!list)
 		{
-			error << "No list has been defined for title '" << val << "'!" << endl;
+			ERROR << "No list has been defined for title '" << val << "'!" << endl;
 			return -1;
 		}
 		
@@ -663,12 +670,12 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (!list)
 		{
-			error << "No list has been defined for address '" << val << "'!" << endl;
+			ERROR << "No list has been defined for address '" << val << "'!" << endl;
 			return -1;
 		}
 		if (lc != "")
 		{
-			error << "list address cannot be localized" << endl;
+			ERROR << "list address cannot be localized" << endl;
 			return -1;
 		}
 		
@@ -678,7 +685,7 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (!list)
 		{
-			error << "No list has been defined for address '" << val << "'!" << endl;
+			ERROR << "No list has been defined for address '" << val << "'!" << endl;
 			return -1;
 		}
 		
@@ -688,19 +695,19 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (!list)
 		{
-			error << "No list has been defined for language '" << val << "'!" << endl;
+			ERROR << "No list has been defined for language '" << val << "'!" << endl;
 			return -1;
 		}
 		if (lc != "")
 		{
-			error << "list language cannot be localized" << endl;
+			ERROR << "list language cannot be localized" << endl;
 			return -1;
 		}
 		
 		string lval(val);
 		if (!lstring::lang_normalize(lval))
 		{
-			error << "Language '" << val << "' is not an ISO 639 language code!" << endl;
+			ERROR << "Language '" << val << "' is not an ISO 639 language code!" << endl;
 			error << "Regional variants are not relevant for searches." << endl;
 			return -1;
 		}
@@ -711,12 +718,12 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (!list)
 		{
-			error << "No list has been defined for offline setting '" << val << "'!" << endl;
+			ERROR << "No list has been defined for offline setting '" << val << "'!" << endl;
 			return -1;
 		}
 		if (lc != "")
 		{
-			error << "list offline cannot be localized" << endl;
+			ERROR << "list offline cannot be localized" << endl;
 			return -1;
 		}
 		
@@ -726,7 +733,7 @@ int Config::process_command(const string& keys, const string& val, const string&
 			list->offline = true;
 		else
 		{
-			error << "offline must be set to on/off or true/false!" << endl;
+			ERROR << "offline must be set to on/off or true/false!" << endl;
 			return -1;
 		}
 	}
@@ -734,7 +741,7 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (!list)
 		{
-			error << "No list has been defined for address '" << val << "'!" << endl;
+			ERROR << "No list has been defined for address '" << val << "'!" << endl;
 			return -1;
 		}
 		
@@ -746,13 +753,13 @@ int Config::process_command(const string& keys, const string& val, const string&
 		
 		if (lc != "")
 		{
-			error << "frontend path '" <<  val << "' cannot be localized" << endl;
+			ERROR << "frontend path '" <<  val << "' cannot be localized" << endl;
 			return -1;
 		}
 		
 		if (val.length() == 0 || val[0] != '/')
 		{
-			error << "frontend path is not absolute '" << val << "' (must start with a '/')!" << endl;
+			ERROR << "frontend path is not absolute '" << val << "' (must start with a '/')!" << endl;
 			return -1;
 		}
 		
@@ -763,10 +770,18 @@ int Config::process_command(const string& keys, const string& val, const string&
 		if (frontends.find(sval) == frontends.end())
 		{
 			frontend = &frontends[sval];
+			
+			// Inherit global values
+			frontend->admin_name = admin_name;
+			frontend->admin_address = admin_address;
+			frontend->archive = archive;
+			frontend->hide_email = hide_email;
+			frontend->raw_email = raw_email;
+			frontend->web_cache = web_cache;
 		}
 		else
 		{
-			error << "Frontend '" << val << "' already exists!" << endl;
+			ERROR << "Frontend '" << val << "' already exists!" << endl;
 			return -1;
 		}
 	}
@@ -774,13 +789,13 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (frontend == 0)
 		{
-			error << "No frontend defined for allow_list = '" << val << "'" << endl;
+			ERROR << "No frontend defined for allow_list = '" << val << "'" << endl;
 			return -1;
 		}
 		
 		if (lists.find(val) == lists.end())
 		{
-			error << "List '" << val << "' does not exist for allow_list" << endl;
+			ERROR << "List '" << val << "' does not exist for allow_list" << endl;
 			return -1;
 		}
 		
@@ -794,13 +809,13 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (frontend == 0)
 		{
-			error << "No frontend defined for deny_list = '" << val << "'" << endl;
+			ERROR << "No frontend defined for deny_list = '" << val << "'" << endl;
 			return -1;
 		}
 		
 		if (lists.find(val) == lists.end())
 		{
-			error << "List '" << val << "' does not exist for deny_list" << endl;
+			ERROR << "List '" << val << "' does not exist for deny_list" << endl;
 			return -1;
 		}
 		
@@ -814,13 +829,13 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (frontend == 0)
 		{
-			error << "No frontend defined for allow_group = '" << val << "'" << endl;
+			ERROR << "No frontend defined for allow_group = '" << val << "'" << endl;
 			return -1;
 		}
 		
 		if (groups.find(val) == groups.end())
 		{
-			error << "Group '" << val << "' does not exist for allow_group" << endl;
+			ERROR << "Group '" << val << "' does not exist for allow_group" << endl;
 			return -1;
 		}
 		
@@ -834,13 +849,13 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (frontend == 0)
 		{
-			error << "No frontend defined for deny_group = '" << val << "'" << endl;
+			ERROR << "No frontend defined for deny_group = '" << val << "'" << endl;
 			return -1;
 		}
 		
 		if (groups.find(val) == groups.end())
 		{
-			error << "Group '" << val << "' does not exist for deny_group" << endl;
+			ERROR << "Group '" << val << "' does not exist for deny_group" << endl;
 			return -1;
 		}
 		
@@ -854,33 +869,58 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (lc != "")
 		{
-			error << "dbdir cannot be localized" << endl;
+			ERROR << "dbdir cannot be localized" << endl;
+			return -1;
+		}
+		
+		if (val[0] == '/')
+			dbdir = val;
+		else	dbdir = dir + val;
+	}
+	else if (key == "db_umask")
+	{
+		if (lc != "")
+		{
+			ERROR << "db_umask cannot be localized" << endl;
+			return -1;
+		}
+		
+		char* e;
+		db_umask = strtol(val.c_str(), &e, 8);
+		if (val.length() == 0 || *e != 0)
+		{
+			ERROR << "db_mask must be given an octal number, not '" << val << "'" << endl;
 			return -1;
 		}
-		dbdir = val;
 	}
 	else if (key == "admin_name")
 	{
-		admin_name.translate(lc, val);
+		if (frontend == 0)
+			admin_name.translate(lc, val);
+		else	frontend->admin_name.translate(lc, val);
 	}
 	else if (key == "admin_address")
 	{
 		if (lc != "")
 		{
-			error << "admin_address cannot be localized" << endl;
+			ERROR << "admin_address cannot be localized" << endl;
 			return -1;
 		}
-		admin_address = val;
+		if (frontend == 0)
+			admin_address = val;
+		else	frontend->admin_address = val;
 	}
 	else if (key == "archive")
 	{
-		archive.translate(lc, val);
+		if (frontend == 0)
+			archive.translate(lc, val);
+		else	frontend->archive.translate(lc, val);
 	}
 	else if (key == "xslt")
 	{
 		if (lc != "")
 		{
-			error << "xslt command cannot be localized" << endl;
+			ERROR << "xslt command cannot be localized" << endl;
 			return -1;
 		}
 		xslt = val;
@@ -889,7 +929,7 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (lc != "")
 		{
-			error << "delete_message command cannot be localized" << endl;
+			ERROR << "delete_message command cannot be localized" << endl;
 			return -1;
 		}
 		delete_message = val;
@@ -898,7 +938,7 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (lc != "")
 		{
-			error << "pgp_verify_mime command cannot be localized" << endl;
+			ERROR << "pgp_verify_mime command cannot be localized" << endl;
 			return -1;
 		}
 		pgpv_mime = val;
@@ -907,7 +947,7 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (lc != "")
 		{
-			error << "pgp_verify_inline command cannot be localized" << endl;
+			ERROR << "pgp_verify_inline command cannot be localized" << endl;
 			return -1;
 		}
 		pgpv_inline = val;
@@ -916,58 +956,76 @@ int Config::process_command(const string& keys, const string& val, const string&
 	{
 		if (lc != "")
 		{
-			error << "web_cache cannot be localized" << endl;
+			ERROR << "web_cache cannot be localized" << endl;
 			return -1;
 		}
+		
+		bool newval;
 		if (val == "off" || val == "false")
-			web_cache = false;
+			newval = false;
 		else if (val == "on" || val == "true")
-			web_cache = true;
+			newval = true;
 		else
 		{
-			error << "web_cache must be set to on/off or true/false!" << endl;
+			ERROR << "web_cache must be set to on/off or true/false!" << endl;
 			return -1;
 		}
+		
+		if (frontend == 0)
+			web_cache = newval;
+		else	frontend->web_cache = newval;
 	}
 	else if (key == "hide_email")
 	{
 		if (lc != "")
 		{
-			error << "hide_email cannot be localized" << endl;
+			ERROR << "hide_email cannot be localized" << endl;
 			return -1;
 		}
+		
+		bool newval;
 		if (val == "off" || val == "false")
-			hide_email = false;
+			newval = false;
 		else if (val == "on" || val == "true")
-			hide_email = true;
+			newval = true;
 		else
 		{
-			error << "hide_email must be set to on/off or true/false!" << endl;
+			ERROR << "hide_email must be set to on/off or true/false!" << endl;
 			return -1;
 		}
+		
+		if (frontend == 0)
+			hide_email = newval;
+		else	frontend->hide_email = newval;
 	}
 	else if (key == "raw_email")
 	{
 		if (lc != "")
 		{
-			error << "raw_email cannot be localized" << endl;
+			ERROR << "raw_email cannot be localized" << endl;
 			return -1;
 		}
+		
+		bool newval;
 		if (val == "off" || val == "false")
-			raw_email = false;
+			newval = false;
 		else if (val == "on" || val == "true")
-			raw_email = true;
+			newval = true;
 		else
 		{
-			error << "raw_email must be set to on/off or true/false!" << endl;
+			ERROR << "raw_email must be set to on/off or true/false!" << endl;
 			return -1;
 		}
+		
+		if (frontend == 0)
+			raw_email = newval;
+		else	frontend->raw_email = newval;
 	}
 	else if (key == "include")
 	{
 		if (lc != "")
 		{
-			error << "include cannot be localized" << endl;
+			ERROR << "include cannot be localized" << endl;
 			return -1;
 		}
 		string file;
@@ -976,18 +1034,52 @@ int Config::process_command(const string& keys, const string& val, const string&
 			file = val;
 		else	file = dir + val;
 		
-		if (load(file, false) != 0)
-			return -1;
+		DIR* d = opendir(file.c_str());
+		if (d)
+		{
+			struct dirent* e;
+			struct stat s;
+			vector<string> paths;
+			while ((e = readdir(d)) != 0)
+			{
+				int len = strlen(e->d_name);
+				if (e->d_name[0] == '.') continue;
+				if (len <= 5) continue;
+				if (strcmp(".conf", &e->d_name[len-5]))
+					continue;
+				
+				string path = file + '/' + e->d_name;
+				if (stat(path.c_str(), &s) != 0) continue;
+				if (!S_ISREG(s.st_mode)) continue;
+				
+				paths.push_back(path);
+			}
+			closedir(d);
+			
+			// Include them in sorted order
+			sort(paths.begin(), paths.end());
+			for (vector<string>::const_iterator i = paths.begin();
+			    i != paths.end(); ++i)
+			{
+				if (load(*i, false) != 0)
+					return -1;
+			}
+		}
+		else
+		{
+			if (load(file, false) != 0)
+				return -1;
+		}
 	}
 	else
 	{
-		error << "Unknown configuration directive '" << key << "'!" << endl;
+		ERROR << "Unknown configuration directive '" << key << "'!" << endl;
 		return -1;
 	}
 	
 	if (val.length() > len)
 	{
-		error << "Value '" << val << "' is too long for directive '" << key << "'!" << endl;
+		ERROR << "Value '" << val << "' is too long for directive '" << key << "'!" << endl;
 		return -1;
 	}
 	
@@ -1020,7 +1112,7 @@ ostream& operator << (ostream& o, const List::SerializeMagic& lm)
 	if (m.address.length() > 0)
 		o << " address=\"" << xmlEscape << m.address << "\"";
 	if (m.title.is_set())
-		o << " name=\"" << xmlEscape << m.title(l) << "\"";
+		o << " name=\"" << xmlEscape << whitespace_sanitize(m.title(l)) << "\"";
 	else
 		o << " name=\"" << m.mbox << "\"";
 	
@@ -1064,7 +1156,7 @@ ostream& operator << (ostream& o, const Config::SerializeMagic& cm)
 	if (c.admin_address.length() > 0)
 		o << " address=\"" << xmlEscape << c.admin_address << "\"";
 	if (c.admin_name.is_set())
-		o << " name=\"" << xmlEscape << c.admin_name(l) << "\"";
+		o << " name=\"" << xmlEscape << whitespace_sanitize(c.admin_name(l)) << "\"";
 	o << "/></server>";
 	
 	return o;
@@ -1104,4 +1196,12 @@ void Config::set_permissions(const Frontend& f)
 			}
 		}
 	}
+	
+	// Take the specific settings for the globals
+	archive = f.archive;
+	admin_name = f.admin_name;
+	admin_address = f.admin_address;
+	hide_email = f.hide_email;
+	raw_email = f.raw_email;
+	web_cache = f.web_cache;
 }
