@@ -1,4 +1,4 @@
-/*  $Id: ConfigFile.h,v 1.13 2006/02/25 01:05:40 terpstra Exp $
+/*  $Id: ConfigFile.h 1649 2009-10-19 14:35:01Z terpstra $
  *  
  *  ConfigFile.h - Knows how to load the config file
  *  
@@ -115,7 +115,14 @@ struct Frontend
 		string key;
  	};
  	
- 	vector<Entry> entries;
+ 	vector<Entry>	entries;
+ 	
+ 	lstring		archive;
+ 	lstring		admin_name;
+ 	string		admin_address;
+ 	bool		hide_email;
+ 	bool		raw_email;
+ 	bool		web_cache;
 };
 
 class Config
@@ -152,6 +159,7 @@ class Config
         
  	// never localize paths, commands, or addresses
  	string	dbdir;
+ 	int	db_umask;
  	string	xslt;
  	string  delete_message;
  	string	pgpv_mime;
@@ -189,7 +197,7 @@ class Config
  	
  	// Open the config from this file.
  	int load(const string& file, bool toplevel = true);
- 	int process_command(const string& key, const string& val, const string& dir);
+ 	int process_command(const string& file, int c, const string& key, const string& val, const string& dir);
  	
  	// Set the allowed flag on lists
  	void set_permissions(const Frontend& f);
