@@ -1,4 +1,4 @@
-/*  $Id: Summary.h,v 1.3 2006/02/21 19:45:45 terpstra Exp $
+/*  $Id: Summary.h 1649 2009-10-19 14:35:01Z terpstra $
  *  
  *  Summary.h - Helper which can load a message given MessageId
  *  
@@ -40,6 +40,9 @@ using std::set;
 using std::ostream;
 using namespace ESort;
 
+// Convert all linear white space into a single space
+string whitespace_sanitize(const string& x);
+
 class Summary
 {
  protected:
