@@ -1,4 +1,4 @@
-/*  $Id: Index.cpp,v 1.35 2006/02/24 13:17:29 terpstra Exp $
+/*  $Id: Index.cpp 1649 2009-10-19 14:35:01Z terpstra $
  *  
  *  index.cpp - Insert all the keywords from the given email
  *  
@@ -41,8 +41,10 @@
 #include <CharsetEscape.h>
 #include <Keys.h>
 #include <md5.h>
+#include <cstdlib>
 
 #include "Index.h"
+#include "Summary.h"
 
 #include <string>
 #include <vector>
@@ -73,6 +75,23 @@ void utf8Truncate(string& str, string::size_type len)
 	str.resize(len);
 }
 
+static inline char my_toupper(char x)
+{
+	if (x >= 'a' && x <= 'z')
+		return x - 'a' + 'A';
+	else	return x;
+}
+
+static bool strings_equal_case_ignored(const string& a, const string& b)
+{
+	if (a.length() != b.length()) return false;
+	
+	for (string::size_type i = 0; i < a.length(); ++i)
+		if (my_toupper(a[i]) != my_toupper(b[i]))
+			return false;
+	return true;
+}
+
 // first = address, second = name
 pair<string, string> pickAddress(DwAddress* a, const char* charset)
 {
@@ -158,7 +177,7 @@ int Index::index_author()
 		author_name  = addr.second;
 		
 		// Some evil mailing lists set reply-to the list.
-		if (author_email == list.address)
+		if (strings_equal_case_ignored(author_email, list.address))
 		{
 			author_email = "";
 			author_name = "";
@@ -190,6 +209,7 @@ int Index::index_author()
 		if (!author_name .length()) author_name  = addr.second;
 	}
 	
+	author_name = whitespace_sanitize(author_name);
 	utf8Truncate(author_name, 100);
 	//  - nothing longer than 128 could get here (from above)
 	//  - one can never safely truncate an email address
