@@ -1,4 +1,4 @@
-/*  $Id: main.cpp,v 1.49 2006/03/01 14:11:05 terpstra Exp $
+/*  $Id: main.cpp 1649 2009-10-19 14:35:01Z terpstra $
  *  
  *  main.cpp - Read the fed data into our database
  *  
@@ -38,6 +38,7 @@
 #include <cstdio>
 #include <ctime>
 #include <cstdlib>
+#include <cstring>
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -633,6 +634,8 @@ int main(int argc, char** argv)
 		return LEX_DATAERR;
 	}
 	
+	if (cfg.db_umask != -1) umask(cfg.db_umask);
+	
 	string dbname = cfg.dbdir + "/db";
 	ESort::Parameters params(synced);
 	// work around g++ 2.95 borkage
