@@ -1,4 +1,4 @@
-/*  $Id: keyword.cpp,v 1.17 2006/03/01 16:00:01 terpstra Exp $
+/*  $Id: keyword.cpp 1649 2009-10-19 14:35:01Z terpstra $
  *  
  *  jump.cpp - Jump to a given date offset
  *  
@@ -31,6 +31,8 @@
 #include <vector>
 
 #include <ctime>
+#include <cstdlib>
+#include <cstring>
 
 using std::set;
 
