@@ -1,4 +1,4 @@
-/*  $Id: main.cpp,v 1.29 2006/03/02 21:28:50 terpstra Exp $
+/*  $Id: main.cpp 1649 2009-10-19 14:35:01Z terpstra $
  *  
  *  main.cpp - Transform a database snapshot to useful output
  *  
@@ -50,7 +50,7 @@ void error(
 	const string& header)
 {
 	cout << "Status: 200 OK\r\n";
-	cout << header;
+	cout << header; /* optional additional header */
 	cout <<	"Content-Type: text/html\r\n\r\n";
 	cout << "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\r\n"
 	     << "<html>\r\n"
