@@ -74,3 +74,36 @@ function form_timezone(form)
 	d.setFullYear(form.year.value);
 	form.utc.value = d.valueOf() / 1000;
 }
+
+function readCookie(name)
+{
+	var nameEQ = name + "=";
+	var ca = document.cookie.split(';');
+	for(var i=0;i < ca.length;i++)
+	{
+		var c = ca[i];
+		while (c.charAt(0)==' ') c = c.substring(1,c.length);
+		if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length,c.length);
+	}
+	return null;
+}
+
+function trash(kill)
+{
+	if (readCookie('lurker-pass') == null)
+	{
+		var pass = prompt('Enter the admin password to confirm deletion');
+		if (pass != null)
+		{
+			document.cookie = "lurker-pass="+escape(pass)+"; path=/";
+			self.location = kill;
+		}
+	}
+	else
+	{
+		if (confirm('Really delete this message?'))
+		{
+			self.location = kill;
+		}
+	}
+}
